<?php

function fwembed_parse_html($url = null) {
    if ($url === null) {
    throw new ValueError("Missing URL");
  }


  $ch = curl_init();

  // More complete browser-like headers
  $headers = [
    'Accept: text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8',
    'Accept-Language: en-US,en;q=0.5',
    'Connection: keep-alive',
    'Upgrade-Insecure-Requests: 1',
    'Cache-Control: max-age=0'
  ];

  curl_setopt($ch, CURLOPT_URL, $url);
  curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
  curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
  curl_setopt($ch, CURLOPT_ENCODING, "");
  curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
  curl_setopt($ch, CURLOPT_USERAGENT, 'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:109.0) Gecko/20100101 Firefox/115.0');
  curl_setopt($ch, CURLOPT_TIMEOUT, 30);
  curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false); // Only if necessary for testing
  curl_setopt($ch, CURLOPT_COOKIEJAR, '/tmp/cookies.txt'); // Store cookies
  curl_setopt($ch, CURLOPT_COOKIEFILE, '/tmp/cookies.txt'); // Use cookies

  $html_content = curl_exec($ch);

  if (curl_errno($ch)) {
    $error = curl_error($ch);
    curl_close($ch);
    return "Error fetching URL: " . $error;
  }

  $status_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
  if ($status_code == 403) {
    curl_close($ch);
    return "Access forbidden (403). The website may be blocking automated requests.";
  }

  curl_close($ch);

  $html = null;
  libxml_use_internal_errors(true);
  $dom = new DOMDocument();
  @$dom->loadHTML(loadHTML5($html_content), LIBXML_HTML_NOIMPLIED | LIBXML_HTML_NODEFDTD);
  $dom->documentURI = $url;
  $divs = $dom->getElementsByTagName('div');
  foreach ($divs as $div) {
    if ($div->hasAttribute('data-testid') && $div->getAttribute('data-testid') === 'product') {
      $xpath = new DOMXPath($dom);

      $tileLink = $xpath->query('.//a[contains(@class, "tile")]', $div);
      $tileItem = $xpath->query('.//img[contains(@class, "tile__item--1")
                                and not(contains(@class, "badge"))
                                and not(contains(@class, "tile_options"))
                                and not(contains(@class, "tile__item--2"))]', $div);
      $tileDesc = $xpath->query('.//*[contains(@class, "tile__description")
                                and not(contains(@class, "badge"))
                                and not(contains(@class, "tile_options"))]', $div);
      $tilePrices = $xpath->query('.//*[contains(@class, "tile__prices")
                                  and not(contains(@class, "badge"))
                                  and not(contains(@class, "tile_options"))
                                  and not(contains(@class, "tile__price tile__price--original"))]', $div);
      $productHTML = '';
      $linkHref = '';
      if ($tileLink->length > 0) {
        $linkHref = $tileLink->item(0)->getAttribute('href');
      }
      if ($tileItem->length > 0) {
        $productHTML .= $dom->saveHTML($tileItem->item(0));
      }
      if ($tileDesc->length > 0) {
        $productHTML .= $dom->saveHTML($tileDesc->item(0));
      }
      //if ($tilePrices->length > 0) {
      //  $productHTML .= $dom->saveHTML($tilePrices->item(0));
     // }

      $html = $html . '<div class="product-tile"><a class="product-link" target="_blank" href="' . $url . $linkHref . '">' . $productHTML . '</a></div>';
    }
  }
  libxml_clear_errors();
  return $html;
}

function loadHTML5($html) {
  return '<!DOCTYPE html><html><body>' . $html . '</body></html>';
}

function fwembed_shortcode( $atts ) {
  $options = get_option( 'fourthwall_settings_name' );
  $value = isset( $options['fourth_url'] ) ? $options['fourth_url'] : 'https://fourthwall.com';
  $store_html = fwembed_parse_html($value);
  $store_render = '<div class="fw-store-parent">' . PHP_EOL . $store_html . PHP_EOL . '</div>';
  return $store_render;
}

function fwembed_parse_html_single($url = null, $show_description = false) {
    if ($url === null) {
        throw new ValueError("Missing URL");
    }

    $ch = curl_init();

    // More complete browser-like headers
    $headers = [
        'Accept: text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8',
        'Accept-Language: en-US,en;q=0.5',
        'Connection: keep-alive',
        'Upgrade-Insecure-Requests: 1',
        'Cache-Control: max-age=0'
    ];

    curl_setopt($ch, CURLOPT_URL, $url);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
    curl_setopt($ch, CURLOPT_ENCODING, "");
    curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
    curl_setopt($ch, CURLOPT_USERAGENT, 'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:109.0) Gecko/20100101 Firefox/115.0');
    curl_setopt($ch, CURLOPT_TIMEOUT, 30);
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false); // Only if necessary for testing
    curl_setopt($ch, CURLOPT_COOKIEJAR, '/tmp/cookies.txt'); // Store cookies
    curl_setopt($ch, CURLOPT_COOKIEFILE, '/tmp/cookies.txt'); // Use cookies
    
    $html_content = curl_exec($ch);

    if (curl_errno($ch)) {
        $error = curl_error($ch);
        curl_close($ch);
        return "Error fetching URL: " . $error;
    }

    $status_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
    if ($status_code == 403) {
        curl_close($ch);
        return "Access forbidden (403). The website may be blocking automated requests.";
    }

    curl_close($ch);

    $html = null;
    libxml_use_internal_errors(true);
    $dom = new DOMDocument();
    @$dom->loadHTML(loadHTML5($html_content), LIBXML_HTML_NOIMPLIED | LIBXML_HTML_NODEFDTD);
    $dom->documentURI = $url;
    $xpath = new DOMXPath($dom);
    
    // Extract product information
    $productTitle = $xpath->query('//h1[@class="product-info__title"]');
    $productPrice = $xpath->query('//span[@class="product-info__price product-info__price--original"]');
    $productImage = $xpath->query('//div[@data-gallery="gallery-slide"][1]//img[@class="gallery__image-object"]');
    $productDesc = $xpath->query('//div[@class="product-info__description"]//div[@class="html-formatter"]');
    
    if ($productTitle->length > 0 && $productImage->length > 0) {
        $title = $productTitle->item(0)->textContent;
        $price = $productPrice->length > 0 ? $productPrice->item(0)->textContent : '';
        
        // Get image attributes
        $imageNode = $productImage->item(0);
        $imageSrc = $imageNode->getAttribute('src');
        $imageAlt = $imageNode->getAttribute('alt');
        
        // Build the HTML
        $html = '<div class="product-tile">';
        $html .= '<a class="product-link" target="_blank" href="' . $url . '">';
        $html .= '<img class="tile__item tile__item--1" src="' . $imageSrc . '" alt="' . htmlspecialchars($imageAlt) . '">';
        
        $html .= '<div class="tile__description">';
        $html .= '<h3 class="tile__heading">' . htmlspecialchars(trim($title)) . '</h3>';
        $html .= '<div class="tile__prices">';
        $html .= '<span class="tile__price tile__price--original">' . trim($price) . '</span>';
        $html .= '</div>';
        $html .= '</div>';
        $html .= '</a>';
        
        // Add description if show_description is true
        if ($show_description && $productDesc->length > 0) {
            $description = $dom->saveHTML($productDesc->item(0));
            $html .= '<div class="product-description">' . $description . '</div>';
        }
        
        $html .= '</div>';
    }
    
    libxml_clear_errors();
    return $html;
}

function fwembed_single_shortcode($atts) {
    $atts = shortcode_atts(
        array(
            'url' => '',
            'show_description' => 'false',
        ),
        $atts
    );
    
    if (empty($atts['url'])) {
        return '<p>Error: URL is required for [fourthwall_single] shortcode</p>';
    }
    
    // Convert string 'true'/'false' to boolean
    $show_description = filter_var($atts['show_description'], FILTER_VALIDATE_BOOLEAN);
    
    $product_html = fwembed_parse_html_single($atts['url'], $show_description);
    return '<div class="fw-single-product">' . PHP_EOL . $product_html . PHP_EOL . '</div>';
}
add_shortcode('fourthwall_single', 'fwembed_single_shortcode');
add_shortcode( 'fourthwall', 'fwembed_shortcode' );
